#ifndef MESSAGE_H
#define MESSAGE_H

#include <X11/Xlib.h>

class Button;
class ChooseButton;
class StringGadget;
class ListView;
class ListViewContent;
class GUIElement;
class CycleButton;

typedef struct message
{
  Window window;
  int type;
  KeySym key;
  unsigned int keystate;
  unsigned int button;
  int mousex,mousey;
  int mouserx,mousery;
  int gadgettype; // brauchen wir eigentlich nicht mehr, da jedes GUIElement einen Typ hat
  void *gadget;  // kann auch GUIElement *gadget werden
  Time time;
} Message;

enum {AG_NONE,AG_KEYPRESSED,AG_KEYRELEASED,AG_MOUSEPRESSED,AG_MOUSERELEASED,AG_MOUSECLICKED,AG_MOUSEMOVE,
      AG_EXPOSE,AG_CLOSEWINDOW,AG_ENTERWINDOW,AG_LEAVEWINDOW,AG_BUTTONPRESSED,AG_BUTTONRELEASED,
      AG_BUTTONCLICKED,AG_CHOOSEPRESSED,AG_CHOOSERELEASED,AG_CHOOSECLICKED,
      AG_STRINGGADGET_ACTIVATE,AG_STRINGGADGET_DEACTIVATE,AG_STRINGGADGET_OK,
      AG_STRINGGADGET_CANCEL,AG_STRINGGADGET_CONTENTCHANGE,AG_STRINGGADGET_CURSORCHANGE,
      AG_LISTVIEW_ONESELECT,AG_LISTVIEW_MULTISELECT,AG_LISTVIEW_PRESSED,
      AG_SIZECHANGED,
      AG_DND_START,AG_DND_END,
      AG_CYCLEBUTTONCLICKED};

typedef struct agkey {
  int type;
  Window window;
  KeySym key;
  unsigned int keystate;
  char keyname[10];
} AGKey;

typedef AGKey AGKeyPressed;
typedef AGKey AGKeyReleased;

typedef struct agmouse {
  int type;
  Window window;
  unsigned int button;
  int x,y;
  Time time;
} AGMouse;

typedef AGMouse AGMousePressed;
typedef AGMouse AGMouseReleased;
typedef AGMouse AGMouseClicked;
typedef AGMouse AGMouseMove;

typedef struct agexpose {
  int type;
  Window window;
  int x,y;
  int w,h;
} AGExpose;

typedef struct agclosewindow {
  int type;
  Window window;
} AGCloseWindow;

typedef struct agcross {
  int type;
  Window window;
} AGCross;

typedef AGCross AGEnterWindow;
typedef AGCross AGLeaveWindow;

typedef struct agbutton {
  int type;
  Button *button;
  int state;
} AGButton;

typedef AGButton AGButtonPressed;
typedef AGButton AGButtonReleased;
typedef AGButton AGButtonClicked;

typedef struct agchoose {
  int type;
  ChooseButton *button;
  bool state;
} AGChoose;

typedef AGChoose AGChoosePressed;
typedef AGChoose AGChooseReleased;
typedef AGChoose AGChooseClicked;

typedef struct agstringgadget {
  int type;
  StringGadget *sg;
} AGStringGadget;

typedef AGStringGadget AGStringGadgetActivate;
typedef AGStringGadget AGStringGadgetDeactivate;
typedef AGStringGadget AGStringGadgetOk;
typedef AGStringGadget AGStringGadgetCancel;
typedef AGStringGadget AGStringGadgetContentChange;
typedef AGStringGadget AGStringGadgetCursorChange;

typedef struct aglistview {
  int type;
  ListView *lv;
  ListViewContent *lvc;
  Time time;
} AGListView;

typedef AGListView AGListViewOneSelect;
typedef AGListView AGListViewMultiSelect;
typedef AGListView AGListViewPressed;

typedef struct agsizechange {
  int type;
  Window window;
  int neww,newh;
} AGSizeChange;

typedef struct agdnd {
  int type;
  int relx,rely;
  GUIElement *element;
  void *specialinfo;
} AGDND;

typedef AGDND AGDNDSTART;
typedef AGDND AGDNDEND;

typedef struct agcyclebutton {
  int type;
  CycleButton *cyclebutton;
  int option;
} AGCycleButton;

typedef AGCycleButton AGCycleButtonClicked;

typedef union agmessage {
  int type;
  AGKey key;
  AGMouse mouse;
  AGExpose expose;
  AGCloseWindow closewindow;
  AGCross cross;
  AGButton button;
  AGChoose choose;
  AGStringGadget stringgadget;
  AGListView listview;
  AGSizeChange size;
  AGDND dnd;
  AGCycleButton cyclebutton;
  long direct[20];
} AGMessage;

#endif

