#ifndef COPYOP_H
#define COPYOP_H

#include "wdefines.h"
#include "functionproto.h"
#include <aguix/awindow.h>
#include <aguix/text.h>
#include <aguix/solidbutton.h>
#include <aguix/button.h>
#include <aguix/cyclebutton.h>

class Worker;

class CopyOp:public FunctionProto
{
public:
  CopyOp(Worker*);
  virtual ~CopyOp();
  virtual CopyOp *duplicate();
  virtual bool isName(const char *);
  virtual const char *getName();
  virtual int run(ActionMessage*);
  virtual bool save(Datei*);
  virtual int load(Datei*);
  virtual const char *getDescription();
  virtual int configure();

  int doconfigure(int mode);

  typedef enum {COPYOP_OVERWRITE_NORMAL=0,COPYOP_OVERWRITE_ALWAYS,COPYOP_OVERWRITE_NEVER} overwrite_t;
  typedef enum {COPYOP_NORMALCOPY=0,COPYOP_FASTCOPY,COPYOP_FASTESTCOPY} fastcopy_t;
  void setFollowSymlinks(bool);
  void setMove(bool);
  void setRename(bool);
  void setSameDir(bool);
  void setRequestDest(bool);
  void setRequestFlags(bool);
  void setOverwrite(overwrite_t);
protected:
  static const char *name;
  // Infos to save
  bool follow_symlinks;
  bool move;
  bool rename;
  bool same_dir;
  bool request_dest;
  bool request_flags;

  overwrite_t overwrite;

  fastcopy_t fastcopy;

  char *cp_com[2]; /* extern cp-progs, [0] for normal copy, [1] for forcing overwrite
                      normally no need to change because the "cp" should everywhere ok, but
                      if the user want to */
  
  // temp variables
  Lister *startlister,*endlister;
  ActionMessage *am;
  
  // temp values filled when request_flags==true
  bool tfollow_symlinks;
  bool tmove;
  bool trename;
  bool tsame_dir;
  bool trequest_dest;
  overwrite_t toverwrite;
  fastcopy_t tfastcopy;
  char *tcp_com[2];

  int normalmodecopy();
  int requestdest(char *defaultstr,char **dest);
};

class CopyOpWin
{
public:
  CopyOpWin(AGUIX*);
  ~CopyOpWin();
  int open(bool slim);
  void close();
  void starttimer(); // called once to set timer
  void stoptimer();  // when user-action is needed
  void conttimer();  // when user-interaction finished
  // called once:
  void set_files_to_copy(long nfiles);
  void set_dirs_to_copy(long ndirs);
  void set_bytes_to_copy(long nbytes);
  // calld after newfile()
  void set_bytes_to_copy_actfile(long nbytes);
  
  void dir_finished();
  void file_finished();
  // called when a block from the actual file is copied
  void add_actbytes_copied(long nbytes);
  // called when start a new file
  void newfile(char *name,char *ndest);
  // for setting a message in the two lines used for display copy source/dest
  void setmessage(char *msg,int line);
  // for dec file/dir-counter (mainly in case of skiped files/dirs
  void dec_file_counter(unsigned long f);
  void dec_dir_counter(unsigned long d);
  void dec_byte_counter(unsigned long b);

  /* should be called in copy-loop
   * does the update of the window AND process X-events
   * returnvalues: 0 - normal exists
   *               1 - cancel/close window pressed */ 
  int redraw();
  void forceredraw();
protected:
  char *source;
  char *dest;
  long copied_bytes_actfile,bytes_actfile;
  long copied_bytes,bytes;
  long copied_files,files;
  long copied_dirs,dirs;
  time_t starttime;
  time_t filestarttime;
  time_t stoptime;
  int sbw;
  
  bool slim;
  
  AGUIX *aguix;
  AWindow *window;
  Text *sourcetext,*desttext,*filetext,*globtext,*files2gotext,*dirs2gotext,*timetext;
  SolidButton *fsb,*gsb;
  Button *cb;
  bool update_sourcetext,update_desttext,update_filetext,update_globtext,update_files2gotext;
  bool update_dirs2gotext;
/* actual displayed infos:
   1.source (only filename)
   2.destination (whole path)
   3.progressbar for actual file
   4.<copied Bytes>/<bytes> @ <rate>
   5.files 2 copy
   6.dirs 2 copy
   7.progressbar for the whole operation
   8.<copied Bytes>/<bytes> @ <rate>
*/
};

#endif
