#include "createsymlinkop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *CreateSymlinkOp::name="CreateSymlinkOp";

CreateSymlinkOp::CreateSymlinkOp(Worker *worker):FunctionProto(worker)
{
  same_dir=false;
  local=false;
  requestflags=false;
}

CreateSymlinkOp::~CreateSymlinkOp()
{
}

CreateSymlinkOp*
CreateSymlinkOp::duplicate()
{
  CreateSymlinkOp *ta=new CreateSymlinkOp(worker);
  ta->same_dir=same_dir;
  ta->local=local;
  ta->requestflags=requestflags;
  return ta;
}

bool
CreateSymlinkOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
CreateSymlinkOp::getName()
{
  return name;
}

int
CreateSymlinkOp::run(ActionMessage *msg)
{
  ListerMode *lm1;
  Lister *l1=worker->getActiveLister();
  if(l1!=NULL) {
    startlister=l1;
    endlister=worker->getOtherLister(startlister);
    am=msg;
    lm1=startlister->getActiveMode();
    if(lm1!=NULL)
      if(lm1->isType("NormalMode")==true)
        normalmodecreatesl();
  }
  return 0;
}

bool
CreateSymlinkOp::save(Datei *fh)
{
  int size=3*Datei::getUCharSize();
  fh->putInt(size);

  fh->putUChar((same_dir==true)?1:0);
  fh->putUChar((local==true)?1:0);
  fh->putUChar((requestflags==true)?1:0);
  return true;
}

int
CreateSymlinkOp::load(Datei *fh)
{
  int chunksize=fh->getInt();
  int cs=Datei::getUCharSize();
  if(chunksize>=(3*cs)) {
    if(fh->getUChar()==1) same_dir=true; else same_dir=false;
    if(fh->getUChar()==1) local=true; else local=false;
    if(fh->getUChar()==1) requestflags=true; else requestflags=false;
    chunksize-=3*cs;
  }
  while(chunksize>0) {
    fh->getUChar();
    chunksize--;
  }
  return true;
}

const char *
CreateSymlinkOp::getDescription()
{
  return catalog.getLocaleCom(36);
}

int
CreateSymlinkOp::normalmodecreatesl()
{
  NormalMode *nm1=NULL,
             *nm2=NULL;
  ListerMode *lm1=NULL;
  struct NM_specialsource *specialsource=NULL;
  bool cont=true;
  char *tstr,*dest=NULL;
  struct NM_createsymlinkorder cslorder;
  bool takesamedir;
  
  if(startlister==NULL) return 1;
  lm1=startlister->getActiveMode();
  if(lm1==NULL) return 1;
  if(lm1->isType("NormalMode")!=true) return 1;
  nm1=(NormalMode*)lm1;
  if(endlister!=NULL) {
    lm1=endlister->getActiveMode();
    if(lm1!=NULL)
      if(lm1->isType("NormalMode")==true)
        nm2=(NormalMode*)lm1;
  }

  if(requestflags==true) {
    if(doconfigure(1)!=0) cont=false;
  } else {
    // set values in t* variables
    tsame_dir=same_dir;
    tlocal=local;
  }
  
  if(cont==true) {
    // detect what to copy
    if(am->mode==am->AM_MODE_ONLYACTIVE)
      cslorder.source=cslorder.NM_ONLYACTIVE;
    else if(am->mode==am->AM_MODE_DNDACTION) {
      // insert DND-element into list
      cslorder.source=cslorder.NM_SPECIAL;
      cslorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=NULL;
      specialsource->lvc=NULL;
      //TODO: specialsource nach am besetzen (je nachdem wir ich das realisiere)
      cslorder.sources->addElement(specialsource);
    } else if(am->mode==am->AM_MODE_SPECIAL) {
      cslorder.source=cslorder.NM_SPECIAL;
      cslorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=(FileEntry*)am->extraInfo;
      specialsource->lvc=NULL;
      cslorder.sources->addElement(specialsource);
    } else
      cslorder.source=cslorder.NM_ALLENTRIES;

    takesamedir=true;
    if(am->mode==am->AM_MODE_DNDACTION) {
      //TODO: Ziellister auslesen und dest auf getActualDir setzen
      takesamedir=false;
    } else if(tsame_dir==false) {
      if(nm2!=NULL) {
        tstr=nm2->getActualDir();
        if(tstr!=NULL) {
          dest=dupstring(tstr);
          takesamedir=false;
        }
      }
    }

    if(takesamedir==true) {
      tstr=nm1->getActualDir();
      if(tstr!=NULL)
        dest=dupstring(tstr);
    }
    
    if(dest!=NULL) {
      cslorder.local=tlocal;
      cslorder.destdir=dest;
      // now start copy process
      nm1->createsymlink(&cslorder);
    }
    if(dest!=NULL) _freesafe(dest);
    if(cslorder.source==cslorder.NM_SPECIAL) {
      if(specialsource!=NULL) _freesafe(specialsource);
      delete cslorder.sources;
    }
  }
  return 0;
}

int
CreateSymlinkOp::configure()
{
  return doconfigure(0);
}

int
CreateSymlinkOp::doconfigure(int mode)
{
  AGUIX *aguix=worker->getAGUIX();
  Button *okb,*cb;
  AWindow *win;
  ChooseButton *sdcb,*gcb,*rfcb=NULL;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  Requester *req;
  char *tstr;
  
  req=new Requester(aguix);
  w=h=10;
  x=y=5;
  win=new AWindow(aguix);

  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(36))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(36));
  win->create(NULL,10,10,w,h,0,tstr);
  _freesafe(tstr);

  sdcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(same_dir==true)?1:0,
                                                catalog.getLocale(315),LABEL_RIGHT,1,0));
  y+=sdcb->getHeight()+5;
  tw=sdcb->getWidth()+10;
  if(tw>w) w=tw;

  gcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(local==true)?1:0,
                                               catalog.getLocale(316),LABEL_RIGHT,1,0));
  y+=gcb->getHeight()+5;
  tw=gcb->getWidth()+10;
  if(tw>w) w=tw;

  if(mode==0) {
    rfcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(requestflags==true)?1:0,
                                                 catalog.getLocale(294),LABEL_RIGHT,1,0));
    y+=rfcb->getHeight()+5;
    tw=rfcb->getWidth()+10;
    if(tw>w) w=tw;
  }

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;
  
  h=y;
  win->resize(w,h);
  win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
        case AG_KEYRELEASED:
          if(win->isParent(msg->key.window,false)==true) {
            switch(msg->key.key) {
              case XK_1:
                sdcb->setState((sdcb->getState()==1)?0:1);
                break;
              case XK_2:
                gcb->setState((gcb->getState()==1)?0:1);
                break;
              case XK_Return:
                endmode=0;
                break;
              case XK_Escape:
                endmode=1;
                break;
            }
          }
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    if(mode==1) {
      tsame_dir=(sdcb->getState()==1)?true:false;
      tlocal=(gcb->getState()==1)?true:false;
    } else {
      same_dir=(sdcb->getState()==1)?true:false;
      local=(gcb->getState()==1)?true:false;
      requestflags=(rfcb->getState()==1)?true:false;
    }
  }
  
  win->close();
  delete req;

  return endmode;
}

void CreateSymlinkOp::setSameDir(bool nv)
{
  same_dir=nv;
}

void CreateSymlinkOp::setLocal(bool nv)
{
  local=nv;
}

void CreateSymlinkOp::setRequestFlags(bool nv)
{
  requestflags=nv;
}

