#include "informationmode.h"
#include "normalmode.h"
#include "locale.h"
#include "wconfig.h"
#include "worker.h"

const char *InformationMode::type="InformationMode";

InformationMode::InformationMode(Lister *parent):ListerMode(parent)
{
  x=0;
  y=0;
  w=0;
  h=0;
  lastactivefe=NULL;
  lv=NULL;
  blanked=false;
}

InformationMode::~InformationMode()
{
  lastactivefe=NULL;
}

void InformationMode::messageHandler(AGMessage *msg)
{
  int tx,ty,tw,th;
  bool ma=false;
  switch(msg->type) {
    case AG_SIZECHANGED:
      parentlister->getGeometry(&tx,&ty,&tw,&th);
      reconf(tx,ty,tw,th);
      break;
  }
  if(ma==true) parentlister->makeActive();
}

void InformationMode::on()
{
  parentlister->getGeometry(&x,&y,&w,&h);
  int side=parentlister->getSide();
  int m;

  lv=(ListView*)parentawindow->add(new ListView(aguix,x,y,w,h,1));
  m=(wconfig->getHBarTop(side)==true)?1:2;
  lv->setHBarState(m);
  m=(wconfig->getVBarLeft(side)==true)?1:2;
  lv->setVBarState(m);
  m=wconfig->getHBarHeight(side);
  lv->setHBarHeight(m);
  m=wconfig->getVBarWidth(side);
  lv->setVBarWidth(m);
  lv->setFont(wconfig->getFont(2+side));
  
  for(m=0;m<19;m++) {
    lv->add(new ListViewContent("",1,1,1,1,0,0,0,0,m));
  }

  parentlister->setActiveMode(this);
  setName();
}

void InformationMode::off()
{
  parentawindow->remove(lv);
  parentlister->setActiveMode(NULL);
  parentlister->setName("");
  lastactivefe=NULL;
}

void InformationMode::reconf(int tx,int ty,int tw,int th)
{
  if((tw!=w)||(th!=h)||(tx!=x)||(ty!=y)) {
    w=tw;
    h=th;
    x=tx;
    y=ty;
    lv->resize(w,h);
    lv->move(x,y);
    update(true);
  }
}

void InformationMode::activate()
{
}

void InformationMode::deactivate()
{
}

bool InformationMode::isType(const char *str)
{
  if(strcmp(str,type)==0) return true; else return false;
}

const char *InformationMode::getType()
{
  return type;
}

int InformationMode::configure()
{
  return 0;
}

void InformationMode::cyclicfunc(cyclicfunc_mode_t mode)
{
  update(false);
}

const char* InformationMode::getName()
{
  return catalog.getLocale(175);
}

int InformationMode::load(Datei *fh)
{
  int chunksize=fh->getInt();
  while(chunksize>0) {
    fh->getUChar();
    chunksize--;
  }
  return 0;
}

bool InformationMode::save(Datei *fh)
{
  int cs=0;

  fh->putInt(cs);
  return false;
}

void InformationMode::setName()
{
  parentlister->setName(catalog.getLocale(175));
}

void InformationMode::reconfig()
{
}

void InformationMode::update(bool force)
{
  NormalMode *nm1=NULL;
  ListerMode *lm1=NULL;
  Lister *ol=NULL;
  List *files;
  NM_specialsource *ss1;
  FileEntry *fe;
  
  ol=parentlister->getWorker()->getOtherLister(parentlister);
  if(ol!=NULL) {
    lm1=ol->getActiveMode();
    if(lm1!=NULL)
      if(lm1->isType("NormalMode")==true)
        nm1=(NormalMode*)lm1;
  }
  
  if(force==true) lastactivefe=NULL;
  
  if(nm1!=NULL) {
    files=new List();
    
    nm1->getSelFiles(files,NormalMode::NM_GETFILES_ONLYACTIVE);
    ss1=(NM_specialsource*)files->getFirstElement();
    if(ss1!=NULL) {
      fe=ss1->entry;
      if(fe!=lastactivefe) {
        lastactivefe=fe;
        if(lastactivefe!=NULL) {
          showinfos();
        }
      }
    } else {
      lastactivefe=NULL;
      for(int i=0;i<lv->getElements();i++) lv->get(i)->setText("");
    }
    
    ss1=(NM_specialsource*)files->getFirstElement();
    while(ss1!=NULL) {
      _freesafe(ss1);
      ss1=(NM_specialsource*)files->getNextElement();
    }
    
    delete files;
  }
}

void InformationMode::showinfos()
{
  FileEntry *fe;
  int x,len,len2;
  char buffer[1024],buffer2[MAXPATHLEN],buffer3[10],rights[3][4],*str;
  bool showno;
  unsigned long size;
  unsigned long blocksize,freesize;
  double dsize,dfree;
  
  float percent;
  struct tm *timeptr;
#ifdef HAVE_SYS_STATVFS_H
  struct statvfs fsstatistic;
#else
  struct statfs fsstatistic;
#endif

  showno=true;
  fe=lastactivefe;
  if(fe!=NULL) {
    showno=false;
    sprintf(buffer,"%s: %s",catalog.getLocale(176),fe->name);
    lv->get(1)->setText(buffer);
    if((fe->isDir()==true)&&(fe->isCorrupt==false)) {
      if(fe->dirsize>=0) size=fe->dirsize;
      else size=fe->size;
    } else size=fe->size;
    sprintf(buffer,"%s: %ld B, %ld KB, %ld MB",catalog.getLocale(177),size,(size+512)/1024,(size+524288)/1048576);
    lv->get(2)->setText(buffer);
    
    if(fe->filetype!=NULL) {
      sprintf(buffer,"%s: %s",catalog.getLocale(178),fe->filetype->getName());
    } else {
      if(fe->isDir()==true) sprintf(buffer,"%s: %s",catalog.getLocale(178),catalog.getLocale(109));
      else sprintf(buffer,"%s: %s",catalog.getLocale(178),wconfig->getnotyettype()->getName());
    }
    lv->get(3)->setText(buffer);
    if(fe->isLink==true) {
      str=fe->getDestination();
      sprintf(buffer,"  %s: %s",catalog.getLocale(179),str);
      _freesafe(str);
      lv->get(4)->setText(buffer);
    } else lv->get(4)->setText("");
    
    str=fe->getSPermStr();
    strncpy(rights[0],str,3);
    rights[0][3]='\0';
    strncpy(rights[1],str+3,3);
    rights[1][3]='\0';
    strncpy(rights[2],str+6,3);
    rights[2][3]='\0';
    len=strlen(catalog.getLocale(180));
    len2=strlen(catalog.getLocale(214));
    for(x=1;x<3;x++) if(strlen(catalog.getLocale(214+x))>(size_t)len2) len2=strlen(catalog.getLocale(214+x));
    sprintf(buffer2,"%%%ds %%%ds %%s",len,len2);
    sprintf(buffer,buffer2,catalog.getLocale(180),catalog.getLocale(214),rights[0]);
    lv->get(6)->setText(buffer);
    sprintf(buffer,buffer2,"",catalog.getLocale(215),rights[1]);
    lv->get(7)->setText(buffer);
    sprintf(buffer,buffer2,"",catalog.getLocale(216),rights[2]);
    lv->get(8)->setText(buffer);

    str=getOwnerString(fe->userid,fe->groupid);
    sprintf(buffer,"%s: %s",catalog.getLocale(184),str);
    _freesafe(str);
    lv->get(9)->setText(buffer);

    len=strlen(catalog.getLocale(162));
    for(x=1;x<3;x++) if(strlen(catalog.getLocale(162+x))>(size_t)len) len=strlen(catalog.getLocale(162+x));

    timeptr=localtime(&(fe->lastaccess));
    str=asctime(timeptr);
    str[strlen(str)-1]=0;
    sprintf(buffer2,"%%%ds: %%s",len);
    sprintf(buffer,buffer2,catalog.getLocale(162),str);
    lv->get(11)->setText(buffer);

    timeptr=localtime(&(fe->lastmod));
    str=asctime(timeptr);
    str[strlen(str)-1]=0;
    sprintf(buffer,buffer2,catalog.getLocale(163),str);
    lv->get(12)->setText(buffer);
    timeptr=localtime(&(fe->lastchange));
    str=asctime(timeptr);
    str[strlen(str)-1]=0;
    sprintf(buffer,buffer2,catalog.getLocale(164),str);
    lv->get(13)->setText(buffer);

    sprintf(buffer,"%s: %s",catalog.getLocale(185),fe->fullname);
    lv->get(15)->setText(buffer);

    if(realpath(fe->fullname,buffer2)!=NULL) {
      sprintf(buffer,"%s: %s",catalog.getLocale(186),buffer2);
    } else {
      sprintf(buffer,"%s: ???",catalog.getLocale(186));
    }
    lv->get(16)->setText(buffer);

#ifdef HAVE_STATVFS
    if(statvfs(fe->fullname,&fsstatistic)==0) {
      blocksize=fsstatistic.f_frsize;
      size=fsstatistic.f_blocks;
      freesize=fsstatistic.f_bavail;
#else
    if(statfs(fe->fullname,&fsstatistic)==0) {
      blocksize=fsstatistic.f_bsize;
      size=fsstatistic.f_blocks;
      freesize=fsstatistic.f_bavail;
#endif
      dfree=(double)freesize*(double)blocksize;
      dsize=(double)size*(double)blocksize;
      if(dsize>0) {
        percent=dfree*100/dsize+.5;
      } else {
        percent=0.0;
      }
      if(dsize>10485760) {
        dsize+=524288;
        dsize/=1048576;
        strcpy(buffer3,"MB");
      } else if(dsize>10240) {
        dsize+=512;
        dsize/=1024;
        strcpy(buffer3,"KB");
      } else {
        strcpy(buffer3,"B");
      }
      if(dfree>10485760) {
        dfree+=524288;
        dfree/=1048576;
        strcpy(buffer2,"MB");
      } else if(dfree>10240) {
        dfree+=512;
        dfree/=1024;
        strcpy(buffer2,"KB");
      } else {
        strcpy(buffer2,"B");
      }
      sprintf(buffer,"%s: %ld %s (%2.0f %%) %s %ld %s",catalog.getLocale(187),(unsigned long)dfree,buffer2,percent,
                                                       catalog.getLocale(188),(unsigned long)dsize,buffer3);
    } else {
      sprintf(buffer,"%s: ???",catalog.getLocale(187));
    }
    lv->get(18)->setText(buffer);
  }
  if(showno==true) {
    // fr alle anderen sind keine Informationen erhltlich
    for(int i=0;i<lv->getElements();i++) lv->get(i)->setText("");
    lv->get(2)->setText(catalog.getLocale(189));
  }
  lv->redraw();
}

